/*
Seconda versione:
Cerco di costruire un vettore di punti 3d che descriva i bordi dell'anello 
con il fine di costruire una mesh vera e prorpia che mi rappresenti l'anello.
il vettore di punti viene costruito nella funzione GenerateStrip che sfrutta
il il codice visto nella versione precedente per piazzare il cubetto per
costruire le giuste trasformazioni di modellazione. Poi si fanno passare i
vertici di una sezione dell'anello nella pipeline di opengl (gluproject) 
e si ottiene le coordinate trasformate nel giusto modo.
*/

#include<stdio.h>
#include<stdlib.h>
#include<GL/glut.h>
#include<math.h>
#include<vector>
#include <vcg/Point3.h>
using namespace vcg;
using namespace std;
// globale per memorizzare l'aspect ratio della finestra correntte; 
// usata da myRedraw, e myReshape
float aspect=1;


vector<Point3d> Strip;

// Inizializzazione dello stato di OpenGL
void myInitGL()
{
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	
}


void GenerateStrip()
{
	const float Radius=3;
	const float BlockSide=3;
	int StepNum=240;
	float StepAngleDeg=360.0/StepNum;
	float StepAngleRad=2.0*3.14159265/StepNum;

	double MM[16],MP[16];
	

	//int OldViewPort[4];
	//glGetIntegerv(GL_VIEWPORT,OldViewPort);
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	glOrtho(-1,1,-1,1,-1,1);
	glGetDoublev(GL_PROJECTION_MATRIX,MP);

	// Inizializzazione della Modelview Matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();	
  	
	// -- Disegno dell'Anello -- 
	// l'anello viene create simmetrico rispetto all'asse z.
	// Per renderlo di moebius basta fare aumentare la rotazione lungo
	// l'asse y, fatta prima di ruotare intorno all'asse z; Tale rotazione 
  // deve essere la meta' di quella intorno all'asse z
	// (cosi' dopo 360 gradi ha fatto 180 gradi)
	
	for(int i=0;i<StepNum;++i)
	{
		glPushMatrix();
		glRotatef(i*StepAngleDeg    , 0,0,1);   // Rotazione Intorno all'asse principale
		glTranslatef(Radius,0,0);
		glRotatef(i*StepAngleDeg/2.0, 0,1,0);   // Avvolgimento della striscia.
	
		// -- Trovo i punti sul bordo della striscia -- 
		Point3d pp;
		glGetDoublev(GL_MODELVIEW_MATRIX,MM);
		// Notare il finto viewport fatto per non far fare alle x e y
		// trasformazioni di viewport (la z purtroppo va risistemata a mano)
		int vp[4]={-1,-1,2,2};
		
		gluProject(-1,0,.25,MM,MP,vp,&pp[0],&pp[1],&pp[2]);
		pp[2]=-2.0f*(pp[2]-.5);
		Strip.push_back(pp);
		
		gluProject(1,0,.25,MM,MP,vp,&pp[0],&pp[1],&pp[2]);
		pp[2]=-2.0f*(pp[2]-.5);
		Strip.push_back(pp);
		
		gluProject(1,0,-.25,MM,MP,vp,&pp[0],&pp[1],&pp[2]);
		pp[2]=-2.0f*(pp[2]-.5);
		Strip.push_back(pp);
		
		gluProject(-1,0,-.25,MM,MP,vp,&pp[0],&pp[1],&pp[2]);
		pp[2]=-2.0f*(pp[2]-.5);
		Strip.push_back(pp);
		
		glPopMatrix();
	}
	glColor3f(1,0,0);

}

// Funzione principale di aggiornamento dello schermo
void myRedrawFunc()
{
	int i;
	
	// Gestione dell'animazione
	float AngularVelocity = 120; // degree per sec;
	float ElapsedSecs = float(glutGet(GLUT_ELAPSED_TIME))/1000.0;
	float CurAngleDeg = AngularVelocity * ElapsedSecs;
	float CurAngleRad = CurAngleDeg/180.0*3.14159265;
	
	//Parametri dell'Anello
	const float Radius=3;
	const float BlockSide=3;
	int StepNum=24;
	float StepAngleDeg=360.0/StepNum;
	float StepAngleRad=2.0*3.14159265/StepNum;
	float InternalRotationSpeed=0.33;
	float BallSpeed=0.5;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	gluPerspective(45,aspect,5,15);

	// Inizializzazione della Modelview Matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();	
  
	// World To Camera Transformation
	gluLookAt(0,0,10,0,0,0,0,1,0);
	
	
	glRotatef(CurAngleDeg*0.1,1,0,0);  
	
	if(Strip.empty())
		GenerateStrip();
	glDisable(GL_LIGHTING);
	glBegin(GL_LINE_STRIP);
	for(int i=0;i<Strip.size();++i)
		glVertex(Strip[i]);
	glEnd();
	glEnable(GL_LIGHTING);
	// -- Disegno della Pallina che rotola -- 
	// Assomiglia molto al disegno di un singolo blocchetto dell'anello.
	// a cui va aggiunta una rotazione attorno all'asse x per fare l'effetto
	// di rotolamento della pallina sulla striscia

	glRotatef(CurAngleDeg*BallSpeed,0,0,1);
	glTranslatef(Radius,0,0);
	glRotatef(CurAngleDeg*BallSpeed/2,0,1,0);
		
	glTranslatef(0,0,.8);  // Per spostarla dal piano dell'anello
	glRotatef(-CurAngleDeg*BallSpeed*3,1,0,0); // Rotolamento (la quantita' e' a occhio...)
	
	// -- Disegno della pallina --
	// due sfere rosse/blu leggermente scalate
	glColor3f(1,0,0);
	glutSolidSphere(.6,10,10);
	glColor3f(0,0,1);
	glScalef(1.05f,1.05f,.6f);
	glutSolidSphere(.6,10,10);

	glFlush();
  glutSwapBuffers();
}

void myIdle()
{
	glutPostRedisplay();
}

void myReshapeFunc(int wx, int wy)
{
	glViewport(0,0,wx,wy);
	aspect=float(wx)/float(wy);
}
int main(int argc, char *argv[])
{
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInit(&argc, argv);
	glutIdleFunc(myIdle);
	glutCreateWindow("Moebius");
	glutDisplayFunc(myRedrawFunc);
	glutReshapeFunc(myReshapeFunc);
	myInitGL();
	glutMainLoop(); 	//Passare al SO il controllo.
}